/**
 * A client-side 802.1x implementation supporting EAP/TLS
 *
 * This code is released under both the GPL version 2 and BSD licenses.
 * Either license may be used.  The respective licenses are found below.
 *
 * Copyright (C) 2002 Bryan D. Payne & Nick L. Petroni Jr.
 * All Rights Reserved
 *
 * --- GPL Version 2 License ---
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * --- BSD License ---
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  - Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  - All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *       This product includes software developed by the University of
 *       Maryland at College Park and its contributors.
 *  - Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*******************************************************************
 * The driver function for a Linux application layer EAPOL 
 * implementation
 * File: auth_tools.c
 *
 * Authors: Chris.Hessing@utah.edu
 *
 * $Id: auth_tools.c,v 1.3 2003/02/07 18:46:49 chessing Exp $
 * $Date: 2003/02/07 18:46:49 $
 * $Log: auth_tools.c,v $
 * Revision 1.3  2003/02/07 18:46:49  chessing
 * First import of MS-CHAPv2 code.  Works, but still needs some work done on it.
 *
 * Revision 1.2  2003/01/24 20:48:21  chessing
 * Cleaned up framing code, TLS no longer sends a malformed frame.
 *
 * Revision 1.1  2003/01/23 22:45:00  chessing
 * Added some global functions to handle packet fragments, and more TLS cleanups.
 *
 *
 ****************************************************************/
//
//
// These are utility functions to be used with the authentication methods.
//
//

#include <stdlib.h>
#include <string.h>

#include "auth_tools.h"
#include "logging.h"

char *stored_data_frags = NULL;
int frags_size = 0;
int frags_ptr = 0; 

void print_hex(uint8_t *vals, int vals_length)
{
  int i;

  for (i=0;i<vals_length;i++)
    {
      xlogf(DEBUG_AUTHTYPES, "%x ",vals[i]);
    }
  xlogf(DEBUG_AUTHTYPES, "\n");
}


//
// This function gathers packet fragments and puts them together.
//
int save_data_fragment(char *data_frag, int in_size)
{
  char *new_mem_loc = NULL;

  if (stored_data_frags == NULL)
    {
      // If stored_data_frags is NULL, then we are starting a new buffer,
      // so we need to malloc in_size.
      stored_data_frags = (char *)malloc(in_size);
      
      if (stored_data_frags == NULL)
	{
	  xlogf(DEBUG_NORMAL, "Couldn't allocate memory needed to store data fragments.");
	  return -1;
	}
    }

  // If frags_size == frags_ptr, then we need to malloc a bigger chunk.
  if (frags_size == frags_ptr)
    {
      new_mem_loc = (char *)malloc(frags_size + in_size);

      //Make a copy of what we have so far.
      memcpy(new_mem_loc, stored_data_frags, frags_size);

      frags_size = frags_size + in_size;  // This is the new fragment size.
      
      free(stored_data_frags);            // Clear the old memory
      
      stored_data_frags = new_mem_loc;    // Change the pointer.

      new_mem_loc = NULL;                 // Trash the old pointer.
    }

  memcpy(&stored_data_frags[frags_ptr], data_frag, in_size);
  frags_ptr += in_size;
  return 0;
}

//
// This returns the completed packet fragments.
//
char *get_data_fragment()
{
  return stored_data_frags;
}

int get_data_frag_size()
{
  return frags_size;
}

//
// destroy_data_frags clears the memory in use.
//
void destroy_data_frags()
{
  if (stored_data_frags != NULL) free(stored_data_frags);
  stored_data_frags = NULL;
  frags_size = 0;
  frags_ptr = 0;
}
